
// Define pins for ultrasonic and buzzer
int const trigPin = 10;
int const echoPin = 9;
int const buzzPin = 2;

double lastDistanceMeasured = 80;
int numCyclesUnchanged = 0;
double diff = 0.6;

#include <Adafruit_NeoPixel.h>
#include <avr/power.h>
#include <math.h>

#define PIN 6
#define NUMPIXELS 16

Adafruit_NeoPixel pixels(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);

#define DELAYVAL 500


void setup()
{
  pinMode(trigPin, OUTPUT); // trig pin will have pulses output
  pinMode(echoPin, INPUT); // echo pin should be input to get pulse width
  pinMode(buzzPin, OUTPUT); // buzz pin is output to control buzzering
  pixels.begin();
}

void setFirstNumLights(int num, int red, int green, int blue, int frequency) {
  // Buzz
  tone(2, frequency, 100);
  pixels.clear();
  for (int i = 0; i < 16; i++) {
    if (i < num) {
      pixels.setPixelColor(i, pixels.Color(red, green, blue));
    } else {
      pixels.setPixelColor(i, pixels.Color(0, 0, 0));
    }
  } 
  pixels.show();
}

void activeMonitoring(double distance) {
  if (distance > 69) {                            // 72
    setFirstNumLights(16, 0, 0, 0, 0);
  } else if (distance <= 69 && distance > 65) {  // 72 - 70
    setFirstNumLights(1, 0, 255, 0, 261);
  } else if (distance <= 65 && distance > 61) {  // 69 - 67
    setFirstNumLights(2, 0, 255, 0, 261);
  } else if (distance <= 61 && distance > 57) {  // 66 - 64
    setFirstNumLights(3, 0, 255, 0, 261);
  } else if (distance <= 57 && distance > 53) {  // 63 - 61
    setFirstNumLights(4, 0, 255, 0, 261);
  } else if (distance <= 53 && distance > 49) {  // 60 - 58
    setFirstNumLights(5, 0, 255, 0, 261);
  } else if (distance <= 49 && distance > 45) {  // 57 - 55
    setFirstNumLights(6, 0, 255, 0, 261);
  } else if (distance <= 45 && distance > 41) {  // 54 - 52
    setFirstNumLights(7, 0, 255, 0, 261);
  } else if (distance <= 41 && distance > 37) {  // 51 - 49
    setFirstNumLights(8, 0, 255, 0, 330);
  } else if (distance <= 37 && distance > 33) {  // 48 - 46
    setFirstNumLights(9, 0, 255, 0, 330);
  } else if (distance <= 33 && distance > 29) {  // 45 - 43
    setFirstNumLights(10, 0, 255, 0, 330);
  } else if (distance <= 29 && distance > 25) {  // 42 - 40
    setFirstNumLights(11, 0, 255, 0, 330);
  } else if (distance <= 25 && distance > 21) {  // 39 - 37
    setFirstNumLights(12, 0, 255, 0, 330);
  } else if (distance <= 21 && distance > 17) {  // 36 - 34
    setFirstNumLights(13, 0, 255, 0, 330);
  } else if (distance <= 17 && distance > 13) {  // 33 - 31
    setFirstNumLights(14, 0, 255, 0, 330);
  } else if (distance <= 13 && distance > 9) {  // 30 - 28
    setFirstNumLights(15, 0, 255, 0, 330);
  } else if (distance <= 9 && distance > 5) {  // 27 - 25
    setFirstNumLights(16, 0, 255, 0, 415);
  } else if (distance <= 5 && distance > 0) {   // 24 - 0
    setFirstNumLights(16, 255, 0, 0, 415);
  } else {
    // Don't buzz
    digitalWrite(buzzPin, LOW);
    pixels.clear();
  }
  // Waiting 60 ms won't hurt any one
  delay(60);
}

void passiveMonitoring() {
  pixels.clear();
  setFirstNumLights(16, 0, 0, 0, 0);
  pixels.show();
  digitalWrite(buzzPin, LOW);
  // Wait num seconds (seconds X 1000 = ms)
  int ms = 15 * 1000;
  delay(ms);
}

void loop()
{
  pixels.clear();
  // Duration will be the input pulse width and distance will be the distance to the obstacle in centimeters
  int duration, distance;
  // Output pulse with 1ms width on trigPin
  digitalWrite(trigPin, HIGH); 
  delay(1);
  digitalWrite(trigPin, LOW);
  // Measure the pulse input in echo pin
  duration = pulseIn(echoPin, HIGH, 1000000);
  // Distance is half the duration devided by 29.1 (from datasheet)
  distance = (duration/2) / 29.1;
  // if distance less than 0.5 meter and more than 0 (0 or less means over range) 
  
  diff = lastDistanceMeasured - distance;
  
  if (diff < 0) {
    diff = diff * -1;
  }

  if (diff < 0.5 || distance > 69) {
    numCyclesUnchanged++;
  } else {
    numCyclesUnchanged = 0;
  }

  if (numCyclesUnchanged > 150) {
    // passive monitoring 
    passiveMonitoring();
    if (numCyclesUnchanged > 2000) {
      numCyclesUnchanged = 151;
    }
  } else {
    // active monitoring
    activeMonitoring(distance);
  }
   
  lastDistanceMeasured = distance;
}
